/**
 * source: https://github.com/Nozbe/WatermelonDB/blob/master/src/utils/fp/zip/index.js
 * Creates a new list out of the two supplied by pairing up equally-positioned
 * items from both lists. The returned list is truncated to the length of the
 * shorter of the two input lists.
 *
 * @func
 * @category List
 * @sig [a] -> [b] -> [[a,b]]
 * @param {Array} left The first array to consider.
 * @param {Array} right The second array to consider.
 * @return {Array} The list made by pairing up same-indexed elements of `left` and `right`.
 * @example
 *
 *      R.zip([1, 2, 3], ['a', 'b', 'c']); //=> [[1, 'a'], [2, 'b'], [3, 'c']]
 * @symb R.zip([a, b, c], [d, e, f]) = [[a, d], [b, e], [c, f]]
 */
export function zip(left, right){
  if (arguments.length === 1) return _right => zip(left, _right)

  const result = []
  const length = Math.min(left.length, right.length)

  for (let i = 0; i < length; i++){
    result[ i ] = [ left[ i ], right[ i ] ]
  }

  return result
}
